/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.jdbc.handlers.builtin;

import org.apache.xerces.parsers.DOMParser;
import org.apache.xerces.parsers.StandardParserConfiguration;
import org.apache.xerces.xs.XSModel;
import org.eclipse.wsdl.validate.ValidationInfo;
import org.eclipse.wsdl.validate.ValidationMessage;
import org.eclipse.wsdl.validate.internal.ValidationInfoImpl;
import org.eclipse.wsdl.validate.internal.resolver.URIResolver;
import org.eclipse.wsdl.validate.internal.util.MessageGenerator;
import org.eclipse.wsdl.validate.internal.xml.LineNumberDOMParser;
import org.eclipse.wsdl.validate.wsdl11.internal.WSDL11BasicValidator;
import org.eclipse.wsdl.validate.wsdl11.internal.WSDL11ValidationInfoImpl;
import org.eclipse.wsdl.validate.wsdl11.internal.WSDL11ValidatorController;
import org.eclipse.wsdl.validate.wsdl11.internal.WSDLDocument;
import org.w3c.dom.Document;
import org.wso2.registry.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.jdbc.handlers.Handler;
import org.wso2.registry.jdbc.handlers.RequestContext;
import org.xml.sax.InputSource;

import javax.wsdl.Definition;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLReader;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.ResourceBundle;

public class WSDLValidationHandler extends Handler {

    private static final String WSDL_VALIDATION_ERROR = "ValidationMessages";

    private static final String WSDL_STATUS = "WSDL Status ";
    private static final String WSDL_VALID = "WSDL is valid";
    private static final String WSDL_IN_VALID = "WSDL is invalid ";

    public Resource get(RequestContext requestContext) throws RegistryException {
        return null;
    }

    public void put(RequestContext requestContext) throws RegistryException {
        validateWSDL(requestContext);
    }

    private void validateWSDL(RequestContext requestContext) throws RegistryException {
        Resource resource = requestContext.getResource();
        Object resourceContent = resource.getContent();
        if (resourceContent instanceof byte[]) {
            try {
                InputStream in = new ByteArrayInputStream((byte[]) resourceContent);
                ResourceBundle rb = ResourceBundle.getBundle("validatewsdl");
                MessageGenerator messagegenerator = new MessageGenerator(rb);

                StandardParserConfiguration configuration = new StandardParserConfiguration();
                DOMParser builder = new LineNumberDOMParser(configuration);
                builder.parse(new InputSource(in));

                DocumentBuilder db;
                DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
                dbf.setNamespaceAware(true);

                try {
                    db = dbf.newDocumentBuilder();
                } catch (Exception e) {
                    dbf = DocumentBuilderFactory.newInstance();
                    db = dbf.newDocumentBuilder();
                }

                Document doc = db.parse(in);
                WSDLReader reader = WSDLFactory.newInstance().newWSDLReader();
                reader.setFeature("javax.wsdl.importDocuments", true);
                Definition wsdlDefinition = reader.readWSDL(getBaseURI(resource.getPath()), doc);
                ValidationInfo validateInfo = new ValidationInfoImpl(resource.getPath(), messagegenerator);
                URIResolver uriResolver = new URIResolver();
                ((ValidationInfoImpl) validateInfo).setURIResolver(uriResolver);
                java.util.Hashtable attributes = new java.util.Hashtable();
                ((ValidationInfoImpl) validateInfo).setAttributes(attributes);


                WSDL11ValidationInfoImpl info = new WSDL11ValidationInfoImpl(validateInfo);
                info.setElementLocations(new java.util.Hashtable());
                WSDL11BasicValidator validator = new WSDL11BasicValidator();
                validator.setResourceBundle(rb);
                ValidationMessage[] messages;

                WSDL11ValidatorController wsdl11ValidatorController = new WSDL11ValidatorController();
                WSDLDocument[] wsdlDocs = wsdl11ValidatorController.readWSDLDocument(doc,
                        validateInfo.getFileURI(),
                        messagegenerator,
                        info);
                WSDLDocument document = wsdlDocs[0];
                List schema = document.getSchemas();
                Iterator xsdIter = schema.iterator();
                while (xsdIter.hasNext()) {
                    info.addSchema((XSModel) xsdIter.next());
                }
                // Set the element locations table.
                info.setElementLocations(document.getElementLocations());
                System.out.println("=========================");
                validator.validate(wsdlDefinition, new ArrayList(), info);
                messages = validateInfo.getValidationMessages();
                if (messages.length >0) {
                    resource.setProperty(WSDL_STATUS , WSDL_IN_VALID);
                }   else {
                    resource.setProperty(WSDL_STATUS , WSDL_VALID);
                }
                for (int i = 0; i < messages.length; i++) {
                    ValidationMessage message = messages[i];
                    String messageString = "[" + message.getLine() + "][" + message.getColumn() + "]"
                            + message.getMessage();
                    resource.addProperty(WSDL_VALIDATION_ERROR, messageString);
                }
                //adding the modififed resource back to the registry
                requestContext.getRegistry().put(resource.getPath() , resource);
            } catch (Exception e) {
                new RegistryException(e.getMessage());
            }
        }
    }

    public void importResource(RequestContext requestContext) throws RegistryException {
        validateWSDL(requestContext);
    }

    public void delete(RequestContext requestContext) throws RegistryException {
    }

    public void putChild(RequestContext requestContext) throws RegistryException {

    }

    public void importChild(RequestContext requestContext) throws RegistryException {
    }

    private static String getBaseURI(String currentURI) {
        try {
            File file = new File(currentURI);
            if (file.exists()) {
                return file.getCanonicalFile().getParentFile().toURI()
                        .toString();
            }
            String uriFragment = currentURI.substring(0, currentURI
                    .lastIndexOf("/"));
            return uriFragment + (uriFragment.endsWith("/") ? "" : "/");
        } catch (IOException e) {
            return null;
        }
    }
}
